@@ -238,7 +238,10 @@ static EContactFieldInfo field_info[] = {
 	BOOLEAN_FIELD (E_CONTACT_IS_LIST,             EVC_X_LIST, "list", N_("List"), FALSE),
 	BOOLEAN_FIELD (E_CONTACT_LIST_SHOW_ADDRESSES, EVC_X_LIST_SHOW_ADDRESSES, "list_show_addresses", N_("List Show Addresses"), FALSE),
 
-	STRING_FIELD (E_CONTACT_REV, EVC_REV, "Rev", N_("Last Revision"), FALSE)
+	/* Last modified time */
+	STRING_FIELD (E_CONTACT_REV, EVC_REV, "Rev", N_("Last Revision"), FALSE),
+	/* URI of the book to which the contact belongs to */
+	STRING_FIELD (E_CONTACT_BOOK_URI, EVC_X_BOOK_URI, "book_uri", N_("Book URI"), FALSE)
 };
 
 #undef LIST_ELEM_STR_FIELD
@@ -1196,7 +1199,7 @@ e_contact_get_property (GObject *object,
 		if (attr)
 			rv = info->struct_getter (contact, attr);
 
-		g_value_set_boxed (value, rv);
+		g_value_take_boxed (value, rv);
 	}
 	else if (info->t & E_CONTACT_FIELD_TYPE_GETSET) {
 		EVCardAttribute *attr = e_contact_get_first_attr (contact, info->vcard_field_name);
@@ -1215,7 +1218,7 @@ e_contact_get_property (GObject *object,
 	else if (info->t & E_CONTACT_FIELD_TYPE_SYNTHETIC) {
 		switch (info->field_id) {
 		case E_CONTACT_NAME_OR_ORG: {
-			char *str;
+			const char *str;
 
 			str = e_contact_get_const (contact, E_CONTACT_FILE_AS);
 			if (!str)
@@ -1525,7 +1528,7 @@ free_const_data (gpointer data, GObject *where_object_was)
  *
  * Return value: Depends on the field's type, owned by the #EContact.
  **/
-const gpointer
+gconstpointer
 e_contact_get_const (EContact *contact, EContactField field_id)
 {
 	gboolean is_string = FALSE;
