@@ -26,6 +26,8 @@
 #include <glib.h>
 #include <glib-object.h>
 
+G_BEGIN_DECLS
+
 #define EVC_ADR             "ADR"
 #define EVC_BDAY            "BDAY"
 #define EVC_CALURI          "CALURI"
@@ -62,6 +64,7 @@
 #define EVC_X_BIRTHDAY         "X-EVOLUTION-BIRTHDAY"
 #define EVC_X_BLOG_URL         "X-EVOLUTION-BLOG-URL"
 #define EVC_X_CALLBACK         "X-EVOLUTION-CALLBACK"
+#define EVC_X_COMPANY          "X-EVOLUTION-COMPANY"
 #define EVC_X_DEST_CONTACT_UID "X-EVOLUTION-DEST-CONTACT-UID"
 #define EVC_X_DEST_EMAIL       "X-EVOLUTION-DEST-EMAIL"
 #define EVC_X_DEST_EMAIL_NUM   "X-EVOLUTION-DEST-EMAIL-NUM"
@@ -83,6 +86,7 @@
 #define EVC_X_WANTS_HTML    	"X-MOZILLA-HTML"
 #define EVC_X_YAHOO         	"X-YAHOO"
 #define EVC_X_GROUPWISE     	"X-GROUPWISE"
+#define EVC_X_BOOK_URI     	"X-EVOLUTION-BOOK-URI"
 
 typedef enum {
 	EVC_FORMAT_VCARD_21,
@@ -186,4 +190,6 @@ gboolean         e_vcard_attribute_has_type         (EVCardAttribute *attr, cons
 char*            e_vcard_escape_string (const char *str);
 char*            e_vcard_unescape_string (const char *str);
 
+G_END_DECLS
+
 #endif /* _EVCARD_H */
