@@ -174,6 +174,13 @@ populate_cache (ECalBackendGroupwise *cbgw)
 			/* Show the progress information */
 			num++;
 			percent = ((float) num/total) * 100;
+		
+			/* FIXME The total obtained from the server is wrong. Sometimes the num can 
+			be greater than the total. The following makes sure that the percentage is not >= 100 */
+ 
+			if (percent > 100)
+				percent = 99; 
+
 			progress_string = g_strdup_printf (_("Loading %s items"), type);
 			e_cal_backend_notify_view_progress (E_CAL_BACKEND (cbgw), progress_string, percent);
 			
@@ -1733,7 +1740,7 @@ e_cal_backend_groupwise_create_object (ECalBackendSync *backend, EDataCal *cal,
 			e_cal_backend_cache_put_component (priv->cache, comp);
 			*calobj = e_cal_component_get_as_string (comp);
 		} else {
-			
+			EGwConnectionStatus status;	
 			GList *list = NULL, *tmp;
 			GPtrArray *uid_array = g_ptr_array_new ();
 			for (l = uid_list; l; l = g_slist_next (l)) {
@@ -1741,12 +1748,16 @@ e_cal_backend_groupwise_create_object (ECalBackendSync *backend, EDataCal *cal,
 			}
 			
 			/* convert uid_list to GPtrArray and get the items in a list */
-			e_gw_connection_get_items_from_ids (priv->cnc,
+			status = e_gw_connection_get_items_from_ids (priv->cnc,
 					priv->container_id, 
 					"attachments recipients message recipientStatus default peek",
 					uid_array, &list);
-			/* FIXME  check if list is null and status may have
-			 * failed. */
+
+			if (status != E_GW_CONNECTION_STATUS_OK || (list == NULL) || (g_list_length (list) == 0)) {
+				g_ptr_array_free (uid_array, TRUE);
+				return GNOME_Evolution_Calendar_OtherError;
+			}
+			
 			comp = g_object_ref ( (ECalComponent *) list->data );
 			/* convert items into components and add them to the cache */
 			for (i=0, tmp = list; tmp ; tmp = g_list_next (tmp), i++) {
@@ -1829,11 +1840,20 @@ e_cal_backend_groupwise_modify_object (ECalBackendSync *backend, EDataCal *cal,
 		}
 
 		if (e_cal_backend_groupwise_utils_check_delegate (comp, e_gw_connection_get_user_email (priv->cnc))) {
+			const char *id = NULL, *recur_key = NULL;
+			
 			item = e_gw_item_new_for_delegate_from_cal (cbgw, comp);
-			status = e_gw_connection_delegate_request (priv->cnc, item, e_gw_item_get_id (item), NULL, NULL, NULL); 
+
+			if (mod == CALOBJ_MOD_ALL) {
+				recur_key = uid;
+			} else {
+				id = e_gw_item_get_id (item);
+			}
+
+			status = e_gw_connection_delegate_request (priv->cnc, item, id, NULL, NULL, recur_key); 
 		
 			if (status == E_GW_CONNECTION_STATUS_INVALID_CONNECTION)
-					status = e_gw_connection_delegate_request (priv->cnc, item, e_gw_item_get_id (item), NULL, NULL, NULL);
+					status = e_gw_connection_delegate_request (priv->cnc, item, id, NULL, NULL, recur_key);
 				
 				if (status != E_GW_CONNECTION_STATUS_OK) {
 					g_object_unref (comp);
