@@ -73,6 +73,8 @@ typedef struct {
 	void (*object) (ECalListener *listener, ECalendarStatus status, const char *object);
 	void (*object_list) (ECalListener *listener, ECalendarStatus status, GList **objects);
 
+	void (*attachment_list) (ECalListener *listener, ECalendarStatus status, GSList **objects);
+	
 	void (*get_timezone) (ECalListener *listener, ECalendarStatus status, const char *object);
 	void (*add_timezone) (ECalListener *listener, ECalendarStatus status, const char *tzid);
 	void (*set_default_timezone) (ECalListener *listener, ECalendarStatus status, const char *tzid);
@@ -82,8 +84,7 @@ typedef struct {
 	
 	void (*query) (ECalListener *listener, ECalendarStatus status, GNOME_Evolution_Calendar_CalView query);
 
-	void (*categories_changed) (ECalListener *listener, ECalendarStatus status, GPtrArray *categories);
-  void (*auth_required)      (ECalListener *listener);
+	void (*auth_required)      (ECalListener *listener);
 	void (*backend_error) (ECalListener *listener, ECalendarStatus status, const char *message);
 } ECalListenerClass;
 
