@@ -451,6 +451,55 @@ e_cal_backend_cache_get_components (ECalBackendCache *cache)
         return list;
 }
 
+/**
+ * e_cal_backend_cache_get_components_by_uid:
+ * @cache: An #ECalBackendCache object.
+ * @uid: ID of the component to retrieve.
+ *
+ * Retrieves a ical components from the cache.
+ *
+ * Return value: The list of calendar components if found, or NULL otherwise.
+ */
+GSList *
+e_cal_backend_cache_get_components_by_uid (ECalBackendCache *cache, const char *uid)
+{
+        char *comp_str;
+        GSList *l;
+	GSList *list = NULL;
+	icalcomponent *icalcomp;
+	ECalComponent *comp = NULL;
+        
+        /* return null if cache is not a valid Backend Cache.  */
+	g_return_val_if_fail (E_IS_CAL_BACKEND_CACHE (cache), NULL);
+        l = e_file_cache_get_objects (E_FILE_CACHE (cache));
+        if (!l)
+                return NULL;
+        for ( ; l != NULL; l = g_slist_next (l)) {
+                comp_str = l->data;
+                if (comp_str) {
+                        icalcomp = icalparser_parse_string (comp_str);
+                        if (icalcomp) {
+				icalcomponent_kind kind;
+
+				kind = icalcomponent_isa (icalcomp);
+				if (kind == ICAL_VEVENT_COMPONENT || kind == ICAL_VTODO_COMPONENT) {
+					comp = e_cal_component_new ();
+					if ((e_cal_component_set_icalcomponent (comp, icalcomp)) &&
+						!strcmp (icalcomponent_get_uid (icalcomp), uid)) 
+							list = g_slist_append (list, comp);
+					else {
+						g_object_unref (comp);
+					}
+				} else
+					icalcomponent_free (icalcomp);
+                        }
+                }
+                
+        }
+
+        return list;
+}
+
 /**
  * e_cal_backend_cache_get_timezone:
  * @cache: An #ECalBackendCache object.
@@ -722,19 +771,14 @@ e_cal_backend_cache_get_marker (ECalBackendCache *cache)
  * Return value: TRUE if the operation was successful, FALSE otherwise.
  */
 gboolean
-e_cal_backend_cache_put_server_utc_time (ECalBackendCache *cache, char *utc_str)
+e_cal_backend_cache_put_server_utc_time (ECalBackendCache *cache, const char *utc_str)
 {
-	char *value;
 	gboolean ret_val = FALSE;
 	
 	g_return_val_if_fail (E_IS_CAL_BACKEND_CACHE (cache), FALSE);
 
-	value = g_strdup (utc_str);
-
-	if (!(ret_val = e_file_cache_add_object (E_FILE_CACHE (cache), "server_utc_time", value)))
-		ret_val = e_file_cache_replace_object (E_FILE_CACHE (cache), "server_utc_time", value);
-
-	g_free (value);
+	if (!(ret_val = e_file_cache_add_object (E_FILE_CACHE (cache), "server_utc_time", utc_str)))
+		ret_val = e_file_cache_replace_object (E_FILE_CACHE (cache), "server_utc_time", utc_str);
 
 	return ret_val;
 }
@@ -753,3 +797,44 @@ e_cal_backend_cache_get_server_utc_time (ECalBackendCache *cache)
 	
        	return	e_file_cache_get_object (E_FILE_CACHE (cache), "server_utc_time");
 }
+
+/**
+ * e_cal_backend_cache_put_key_value:
+ * @cache: An #ECalBackendCache object.
+ * @keyp: The Key parameter to identify uniquely.
+ * @valuep: The value for the keyp parameter.
+ *
+ * Return value: TRUE if the operation was successful, FALSE otherwise.
+ */
+gboolean
+e_cal_backend_cache_put_key_value (ECalBackendCache *cache, const char *key, const char *value)
+{
+	gboolean ret_val = FALSE;
+	
+	g_return_val_if_fail (E_IS_CAL_BACKEND_CACHE (cache), FALSE);
+
+	if (value) {
+		e_file_cache_remove_object (E_FILE_CACHE (cache), key);
+		return TRUE;
+	}
+	
+	if (!(ret_val = e_file_cache_add_object (E_FILE_CACHE (cache), key, value)))
+		ret_val = e_file_cache_replace_object (E_FILE_CACHE (cache), key, value);
+
+	return ret_val;
+}
+
+/**
+ * e_cal_backend_cache_get_key_value:
+ * @cache: An #ECalBackendCache object.
+ *
+ * Return value: The value.
+ */
+const char *
+e_cal_backend_cache_get_key_value (ECalBackendCache *cache, const char *key)
+{
+
+	g_return_val_if_fail (E_IS_CAL_BACKEND_CACHE (cache), NULL);
+	
+       	return	e_file_cache_get_object (E_FILE_CACHE (cache), key);
+}
