@@ -33,7 +33,9 @@
 #undef _
 
 #include <string.h>
+#include <netdb.h>
 #include "camel-i18n.h"
+#include "camel-net-utils.h"
 #include "camel-string-utils.h"
 #include "camel-sasl-kerberos4.h"
 #include "camel-service.h"
