@@ -22,6 +22,7 @@
 
 #include <glib.h>
 #include <camel/camel-mime-message.h>
+#include <camel/camel-string-utils.h>
 #include <e-gw-connection.h>
 #include <e-gw-container.h>
 #include <e-gw-item.h>
@@ -39,7 +40,7 @@
 #define X_TRACK_WHEN            "X-track-when"
 #define X_AUTODELETE            "X-auto-delete"
 #define X_RETURN_NOTIFY_OPEN    "X-return-notify-open"
-#define X_RETURN_NOTIFY_DECLINE "X-return-notify-decline"
+#define X_RETURN_NOTIFY_DELETE  "X-return-notify-delete"
 
 /* Folder types for source */
 #define RECEIVED  "Mailbox"
@@ -70,7 +71,7 @@ gboolean e_path_find_folders (const char *prefix,
 int      e_path_rmdir        (const char *prefix, const char *vpath);
 
 
-EGwItem *camel_groupwise_util_item_from_message (CamelMimeMessage *message, CamelAddress *from, CamelAddress *recipients);
+EGwItem *camel_groupwise_util_item_from_message (EGwConnection *cnc, CamelMimeMessage *message, CamelAddress *from, CamelAddress *recipients);
 void do_flags_diff (flags_diff_t *diff, guint32 old, guint32 _new) ;
 GSList * add_recipients(GSList *recipient_list, CamelAddress *recipients, int recipient_type);
 #endif
