@@ -26,16 +26,16 @@
 #endif
 
 #include <sys/stat.h>
-#include <sys/uio.h>
 #include <unistd.h>
 #include <errno.h>
 #include <string.h>
 #include <stdlib.h>
 
 #include "camel-imap-summary.h"
+#include "camel-imap-utils.h"
 #include "camel-file-utils.h"
 
-#define CAMEL_IMAP_SUMMARY_VERSION (1)
+#define CAMEL_IMAP_SUMMARY_VERSION (3)
 
 static int summary_header_load (CamelFolderSummary *, FILE *);
 static int summary_header_save (CamelFolderSummary *, FILE *);
@@ -43,6 +43,7 @@ static int summary_header_save (CamelFolderSummary *, FILE *);
 static CamelMessageInfo *message_info_load (CamelFolderSummary *s, FILE *in);
 static int message_info_save (CamelFolderSummary *s, FILE *out,
 			      CamelMessageInfo *info);
+static gboolean info_set_user_tag(CamelMessageInfo *info, const char *name, const char  *value);
 static CamelMessageContentInfo *content_info_load (CamelFolderSummary *s, FILE *in);
 static int content_info_save (CamelFolderSummary *s, FILE *out,
 			      CamelMessageContentInfo *info);
@@ -101,6 +102,8 @@ camel_imap_summary_class_init (CamelImapSummaryClass *klass)
 	cfs_class->message_info_save = message_info_save;
 	cfs_class->content_info_load = content_info_load;
 	cfs_class->content_info_save = content_info_save;
+
+	cfs_class->info_set_user_tag = info_set_user_tag;
 }
 
 static void
@@ -145,7 +148,7 @@ static int
 summary_header_load (CamelFolderSummary *s, FILE *in)
 {
 	CamelImapSummary *ims = CAMEL_IMAP_SUMMARY (s);
-
+	
 	if (camel_imap_summary_parent->summary_header_load (s, in) == -1)
 		return -1;
 
@@ -154,10 +157,20 @@ summary_header_load (CamelFolderSummary *s, FILE *in)
 		return camel_file_util_decode_uint32(in, &ims->validity);
 
 	/* Version 1 */
-	if (camel_file_util_decode_fixed_int32(in, &ims->version) == -1
-	    || camel_file_util_decode_fixed_int32(in, &ims->validity) == -1)
+	if (camel_file_util_decode_fixed_int32(in, &ims->version) == -1)
 		return -1;
-
+	
+	if (ims->version == 2) {
+		/* Version 2: for compat with version 2 of the imap4 summary files */
+		int have_mlist;
+		
+		if (camel_file_util_decode_fixed_int32 (in, &have_mlist) == -1)
+			return -1;
+	}
+	
+	if (camel_file_util_decode_fixed_int32(in, &ims->validity) == -1)
+		return -1;
+	
 	if (ims->version > CAMEL_IMAP_SUMMARY_VERSION) {
 		g_warning("Unkown summary version\n");
 		errno = EINVAL;
@@ -180,6 +193,18 @@ summary_header_save (CamelFolderSummary *s, FILE *out)
 	return camel_file_util_encode_fixed_int32(out, ims->validity);
 }
 
+/* We snoop the setting of the 'label' tag, so we can store it on
+   the server, also in a mozilla compatiable way as a bonus */
+static void
+label_to_flags(CamelImapMessageInfo *info)
+{
+	guint32 flags;
+
+	flags = imap_label_to_flags((CamelMessageInfo *)info);
+	if ((info->info.flags & CAMEL_IMAP_MESSAGE_LABEL_MASK) != flags)
+		info->info.flags = (info->info.flags & ~CAMEL_IMAP_MESSAGE_LABEL_MASK) | flags | CAMEL_MESSAGE_FOLDER_FLAGGED;
+}
+
 static CamelMessageInfo *
 message_info_load (CamelFolderSummary *s, FILE *in)
 {
@@ -192,6 +217,8 @@ message_info_load (CamelFolderSummary *s, FILE *in)
 
 		if (camel_file_util_decode_uint32 (in, &iinfo->server_flags) == -1)
 			goto error;
+
+		label_to_flags(iinfo);
 	}
 
 	return info;
@@ -211,6 +238,18 @@ message_info_save (CamelFolderSummary *s, FILE *out, CamelMessageInfo *info)
 	return camel_file_util_encode_uint32 (out, iinfo->server_flags);
 }
 
+static gboolean
+info_set_user_tag(CamelMessageInfo *info, const char *name, const char  *value)
+{
+	int res;
+
+	res = camel_imap_summary_parent->info_set_user_tag(info, name, value);
+
+	if (!strcmp(name, "label"))
+		label_to_flags((CamelImapMessageInfo *)info);
+
+	return res;
+}
 
 static CamelMessageContentInfo *
 content_info_load (CamelFolderSummary *s, FILE *in)
@@ -261,6 +300,8 @@ camel_imap_summary_add_offline (CamelFolderSummary *summary, const char *uid,
 	mi->info.size = camel_message_info_size(info);
 	mi->info.uid = g_strdup (uid);
 
+	label_to_flags(mi);
+
 	camel_folder_summary_add (summary, (CamelMessageInfo *)mi);
 }
 
@@ -272,5 +313,8 @@ camel_imap_summary_add_offline_uncached (CamelFolderSummary *summary, const char
 
 	mi = camel_message_info_clone(info);
 	mi->info.uid = g_strdup(uid);
+
+	label_to_flags(mi);
+
 	camel_folder_summary_add (summary, (CamelMessageInfo *)mi);
 }
