@@ -1,6 +1,8 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*  Camel
- *  Copyright (C) 1999-2004 Jeffrey Stedfast
+/*
+ *  Authors: Jeffrey Stedfast <fejj@novell.com>
+ *
+ *  Copyright 2005 Novell, Inc. (www.novell.com)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,6 +17,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *
  */
 
 
@@ -29,6 +32,7 @@
 
 #include <camel/camel-sasl.h>
 #include <camel/camel-utf8.h>
+#include <camel/camel-stream-process.h>
 #include <camel/camel-tcp-stream-raw.h>
 
 #ifdef HAVE_SSL
@@ -145,6 +149,8 @@ camel_imap4_store_class_init (CamelIMAP4StoreClass *klass)
 	store_class->subscribe_folder = imap4_subscribe_folder;
 	store_class->unsubscribe_folder = imap4_unsubscribe_folder;
 	store_class->noop = imap4_noop;
+
+	/* FIXME: implement folder_subscribed */
 }
 
 static void
@@ -212,6 +218,79 @@ imap4_get_name (CamelService *service, gboolean brief)
 					service->url->user, service->url->host);
 }
 
+
+static gboolean
+connect_to_server_process (CamelIMAP4Engine *engine, const char *format, CamelException *ex)
+{
+	CamelService *service = engine->service;
+	CamelStream *stream;
+	const char *start;
+	GString *exec;
+	
+	exec = g_string_new ("");
+	start = format;
+	
+	while (*format) {
+		register char ch = *format++;
+		
+		if (ch == '%') {
+			g_string_append_len (exec, start, format - start - 1);
+			
+			switch (*format) {
+			case '%':
+				/* literal % */
+				g_string_append_c (exec, '%');
+				break;
+			case 'h':
+				g_string_append (exec, service->url->host);
+				break;
+			case 'u':
+				g_string_append (exec, service->url->user);
+				break;
+			default:
+				g_warning ("unknown formatter '%%%c'", *format);
+				g_string_append_c (exec, '%');
+				g_string_append_c (exec, *format);
+				break;
+			}
+			
+			format++;
+			
+			start = format;
+		}
+	}
+	
+	g_string_append (exec, start);
+	
+	stream = camel_stream_process_new ();
+	
+	if (camel_stream_process_connect ((CamelStreamProcess *) stream, exec->str, NULL) == -1) {
+		if (errno == EINTR)
+			camel_exception_set (ex, CAMEL_EXCEPTION_USER_CANCEL,
+					     _("Connection cancelled"));
+		else
+			camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
+					      _("Could not connect with command \"%s\": %s"),
+					      exec->str, g_strerror (errno));
+		
+		camel_object_unref (stream);
+		g_string_free (exec, TRUE);
+		return FALSE;
+	}
+	
+	g_string_free (exec, TRUE);
+	
+	if (camel_imap4_engine_take_stream (engine, stream, ex) == -1)
+		return FALSE;
+	
+	if (camel_imap4_engine_capability (engine, ex) == -1)
+		return FALSE;
+	
+	camel_imap4_store_summary_set_capabilities (((CamelIMAP4Store *) service)->summary, engine->capa);
+	
+	return TRUE;
+}
+
 enum {
 	MODE_CLEAR,
 	MODE_SSL,
@@ -335,10 +414,15 @@ connect_to_server_wrapper (CamelIMAP4Engine *engine, CamelException *ex)
 	CamelService *service = engine->service;
 	struct addrinfo *ai, hints;
 	const char *ssl_mode;
+	const char *command;
 	int mode, ret, i;
 	const char *port;
 	char *serv;
 	
+	if (camel_url_get_param(service->url, "use_command")
+	    &&(command = camel_url_get_param (service->url, "command")))
+		return connect_to_server_process (engine, command, ex);
+	
 	if ((ssl_mode = camel_url_get_param (service->url, "use_ssl"))) {
 		for (i = 0; ssl_options[i].value; i++)
 			if (!strcmp (ssl_options[i].value, ssl_mode))
@@ -1081,11 +1165,44 @@ imap4_status (CamelStore *store, CamelFolderInfo *fi)
 	g_ptr_array_free (array, TRUE);
 }
 
+static void
+imap4_subscription_info (CamelStore *store, CamelFolderInfo *fi)
+{
+	CamelIMAP4Engine *engine = ((CamelIMAP4Store *) store)->engine;
+	camel_imap4_list_t *lsub;
+	CamelIMAP4Command *ic;
+	GPtrArray *array;
+	char *mailbox;
+	int id, i;
+	
+	mailbox = imap4_folder_utf7_name (store, fi->full_name, '\0');
+	ic = camel_imap4_engine_queue (engine, NULL, "LSUB \"\" %S\r\n", mailbox);
+	camel_imap4_command_register_untagged (ic, "LSUB", camel_imap4_untagged_list);
+	ic->user_data = array = g_ptr_array_new ();
+	
+	while ((id = camel_imap4_engine_iterate (engine)) < ic->id && id != -1)
+		;
+	
+	camel_imap4_command_unref (ic);
+	for (i = 0; i < array->len; i++) {
+		lsub = array->pdata[i];
+		
+		if (!strcmp (lsub->name, mailbox))
+			fi->flags |= CAMEL_FOLDER_SUBSCRIBED;
+		
+		g_free (lsub->name);
+		g_free (lsub);
+	}
+	
+	g_ptr_array_free (array, TRUE);
+}
+
 static CamelFolderInfo *
 imap4_build_folder_info (CamelStore *store, const char *top, guint32 flags, GPtrArray *array)
 {
 	CamelIMAP4Engine *engine = ((CamelIMAP4Store *) store)->engine;
 	CamelFolder *folder = (CamelFolder *) engine->folder;
+	gboolean lsub = (flags & CAMEL_STORE_FOLDER_INFO_SUBSCRIBED);
 	camel_imap4_list_t *list;
 	CamelFolderInfo *fi;
 	char *name, *p;
@@ -1125,10 +1242,13 @@ imap4_build_folder_info (CamelStore *store, const char *top, guint32 flags, GPtr
 		fi->full_name = name;
 		fi->name = g_strdup (p ? p + 1: name);
 		fi->uri = camel_url_to_string (url, CAMEL_URL_HIDE_ALL);
-		fi->flags = list->flags;
+		fi->flags = list->flags | (lsub ? CAMEL_FOLDER_SUBSCRIBED : 0);
 		fi->unread = -1;
 		fi->total = -1;
 		
+		if (!g_ascii_strcasecmp (fi->full_name, "INBOX"))
+			fi->flags |= CAMEL_FOLDER_SYSTEM | CAMEL_FOLDER_TYPE_INBOX;
+		
 		/* SELECTED folder, just get it from the folder */
 		if (folder && !strcmp (folder->full_name, fi->full_name)) {
 			camel_object_get (folder, NULL, CAMEL_FOLDER_TOTAL, &fi->total, CAMEL_FOLDER_UNREAD, &fi->unread, 0);
@@ -1136,12 +1256,20 @@ imap4_build_folder_info (CamelStore *store, const char *top, guint32 flags, GPtr
 			imap4_status (store, fi);
 		}
 		
-		g_free (list->name);
-		g_free (list);
+		if (!(fi->flags & CAMEL_FOLDER_SUBSCRIBED))
+			imap4_subscription_info (store, fi);
 		
 		array->pdata[i] = fi;
 		
 		camel_imap4_store_summary_note_info (((CamelIMAP4Store *) store)->summary, fi);
+		
+		if (!g_ascii_strcasecmp (fi->full_name, "INBOX")) {
+			g_free (fi->name);
+			fi->name = g_strdup (_("Inbox"));
+		}
+		
+		g_free (list->name);
+		g_free (list);
 	}
 	
 	fi = camel_folder_info_build (array, top, '/', TRUE);
@@ -1266,7 +1394,7 @@ imap4_get_folder_info (CamelStore *store, const char *top, guint32 flags, CamelE
 	
 	if (id == -1 || ic->status != CAMEL_IMAP4_COMMAND_COMPLETE) {
 		if (ic1 && ic1->status != CAMEL_IMAP4_COMMAND_COMPLETE)
-			camel_exception_xfer (ex, &ic0->ex);
+			camel_exception_xfer (ex, &ic1->ex);
 		else if (ic0 && ic0->status != CAMEL_IMAP4_COMMAND_COMPLETE)
 			camel_exception_xfer (ex, &ic0->ex);
 		else
