@@ -39,6 +39,10 @@
 #include "e2k-uri.h"
 #include "e2k-utils.h"
 #include "exchange-hierarchy-foreign.h"
+
+/* This is an ugly hack to avoid API break */
+/* Added for get_authtype */
+#include "exchange-esource.h"
 #include <libedataserverui/e-passwords.h>
 
 #include <libgnome/gnome-util.h>
@@ -71,7 +75,7 @@ struct _ExchangeAccountPrivate {
 	GPtrArray *hierarchies;
 	GHashTable *hierarchies_by_folder, *foreign_hierarchies;
 	ExchangeHierarchy *favorites_hierarchy;
-	GHashTable *folders;
+	GHashTable *folders, *fresh_folders;
 	char *uri_authority, *http_uri_schema;
 	gboolean uris_use_email, offline_sync;
 
@@ -154,6 +158,7 @@ init (GObject *object)
 	account->priv->hierarchies_by_folder = g_hash_table_new (NULL, NULL);
 	account->priv->foreign_hierarchies = g_hash_table_new (g_str_hash, g_str_equal);
 	account->priv->folders = g_hash_table_new (g_str_hash, g_str_equal);
+	account->priv->fresh_folders = NULL;
 	account->priv->discover_data_lock = g_mutex_new ();
 	account->priv->account_online = TRUE;
 	account->priv->nt_domain = NULL;
@@ -222,6 +227,12 @@ dispose (GObject *object)
 		account->priv->folders = NULL;
 	}
 
+	if (account->priv->fresh_folders) {
+		g_hash_table_foreach (account->priv->fresh_folders, free_folder, NULL);
+		g_hash_table_destroy (account->priv->fresh_folders);
+		account->priv->fresh_folders = NULL;
+	}
+
 	G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
@@ -310,11 +321,29 @@ void
 exchange_account_rescan_tree (ExchangeAccount *account)
 {
 	int i;
+	EFolder *toplevel;
 
 	g_return_if_fail (EXCHANGE_IS_ACCOUNT (account));
 
-	for (i = 0; i < account->priv->hierarchies->len; i++)
+	if (account->priv->fresh_folders) {
+		g_hash_table_foreach (account->priv->fresh_folders, free_folder, NULL);
+		g_hash_table_destroy (account->priv->fresh_folders);
+		account->priv->fresh_folders = NULL;
+	}
+	account->priv->fresh_folders = g_hash_table_new (g_str_hash, g_str_equal);
+
+	for (i = 0; i < account->priv->hierarchies->len; i++) {
+		/* First include the toplevel folder of the hierarchy as well */
+		toplevel = EXCHANGE_HIERARCHY (account->priv->hierarchies->pdata[i])->toplevel;
+		g_object_ref (toplevel);
+		g_hash_table_insert (account->priv->fresh_folders,
+				     (char *)e_folder_exchange_get_path (toplevel),
+				     toplevel);
+
+		exchange_hierarchy_scan_subtree (account->priv->hierarchies->pdata[i],
+						toplevel, FALSE);
 		exchange_hierarchy_rescan (account->priv->hierarchies->pdata[i]);
+	}
 }
 
 /*
@@ -342,6 +371,14 @@ hierarchy_new_folder (ExchangeHierarchy *hier, EFolder *folder,
 				     folder);
 		table_updated = 1;
 	}
+
+	if (account->priv->fresh_folders) {
+		g_object_ref (folder);
+		g_hash_table_insert (account->priv->fresh_folders,
+				     (char *)e_folder_exchange_get_path (folder),
+				     folder);
+	}	
+	
 	if (!g_hash_table_lookup (account->priv->folders, 
 				e_folder_get_physical_uri (folder))) {
 		/* Avoid dupilcations since the user could add a folder as
@@ -874,6 +911,7 @@ get_password (ExchangeAccount *account, E2kAutoconfig *ac, ExchangeAccountResult
 
 	// SURF : if (exchange_component_is_interactive (global_exchange_component)) {
 		if (!password) {
+	/*
 			char *prompt;
 
 			prompt = g_strdup_printf (_("Enter password for %s"),
@@ -892,6 +930,7 @@ get_password (ExchangeAccount *account, E2kAutoconfig *ac, ExchangeAccountResult
 				account->priv->account->source->save_passwd = remember;
 			}
 			g_free (prompt);
+	*/
 		} 
 		else if (!account->priv->account->source->save_passwd) {
 			/* get_password returns the password cached but user has not 
@@ -915,6 +954,7 @@ get_password (ExchangeAccount *account, E2kAutoconfig *ac, ExchangeAccountResult
  * was due to the password getting expired. If the password has expired
  * this returns TRUE, else it returns FALSE.
  */
+#ifdef HAVE_KRB5
 static gboolean
 is_password_expired (ExchangeAccount *account, E2kAutoconfig *ac)
 {
@@ -947,7 +987,7 @@ is_password_expired (ExchangeAccount *account, E2kAutoconfig *ac)
 
 	return (result == E2K_KERBEROS_PASSWORD_EXPIRED);
 }
-
+#endif
 #if 0
 static void
 change_passwd_cb (GtkWidget *button, ExchangeAccount *account)
@@ -1381,7 +1421,6 @@ exchange_account_connect (ExchangeAccount *account, const char *pword,
 	}
 
 	account->priv->connecting = TRUE;
-	g_mutex_unlock (account->priv->connect_lock);
 
 	if (account->priv->windows_domain)
 		user_name = g_strdup_printf ("%s\\%s", account->priv->windows_domain, account->priv->username);
@@ -1397,12 +1436,20 @@ exchange_account_connect (ExchangeAccount *account, const char *pword,
 	e2k_autoconfig_set_gc_server (ac, account->priv->ad_server,
 				      account->priv->ad_limit);
 
+	if (!pword) {
+		g_mutex_unlock (account->priv->connect_lock);
+		return NULL;
+	}
  try_password_again:
 	if (!pword) {
+/*
 		if (!get_password (account, ac, *info_result)) {
 			account->priv->connecting = FALSE;
 			return NULL;
 		}
+*/
+	} else {
+		e2k_autoconfig_set_password (ac, g_strdup (pword));
 	}
 
  try_connect_again:
@@ -1418,6 +1465,7 @@ exchange_account_connect (ExchangeAccount *account, const char *pword,
 		if ( is_password_expired (account, ac)) {
 			*info_result = EXCHANGE_ACCOUNT_PASSWORD_EXPIRED;
 			account->priv->connecting = FALSE;
+			g_mutex_unlock (account->priv->connect_lock);
 			return NULL;
 /*
 			old_password = exchange_account_get_password (account);
@@ -1447,21 +1495,21 @@ exchange_account_connect (ExchangeAccount *account, const char *pword,
 #endif
 		switch (result) {
 		case E2K_AUTOCONFIG_AUTH_ERROR:
+			*info_result = EXCHANGE_ACCOUNT_PASSWORD_INCORRECT;
 			if (!pword)
 				goto try_password_again;
-
-			*info_result = EXCHANGE_ACCOUNT_PASSWORD_INCORRECT;
 			e2k_autoconfig_free (ac);
 			account->priv->connecting = FALSE;
+			g_mutex_unlock (account->priv->connect_lock);
 			return NULL;
 
 		case E2K_AUTOCONFIG_AUTH_ERROR_TRY_DOMAIN:
+			*info_result = EXCHANGE_ACCOUNT_DOMAIN_ERROR;
 			if (!pword)
 				goto try_password_again;
-
-			*info_result = EXCHANGE_ACCOUNT_DOMAIN_ERROR;
 			e2k_autoconfig_free (ac);
 			account->priv->connecting = FALSE;
+			g_mutex_unlock (account->priv->connect_lock);
 			return NULL;
 
 		case E2K_AUTOCONFIG_AUTH_ERROR_TRY_NTLM:
@@ -1514,12 +1562,13 @@ exchange_account_connect (ExchangeAccount *account, const char *pword,
 			*info_result = EXCHANGE_ACCOUNT_CONNECT_ERROR;
 			break;
 		case E2K_AUTOCONFIG_CANCELLED:
-			return NULL;
+			break;
 		default:
 			*info_result = EXCHANGE_ACCOUNT_UNKNOWN_ERROR;
 			break;
 		}
 
+		g_mutex_unlock (account->priv->connect_lock);
 		return NULL;
 	}
 
@@ -1535,6 +1584,7 @@ exchange_account_connect (ExchangeAccount *account, const char *pword,
 	if (!E2K_HTTP_STATUS_IS_SUCCESSFUL (status)) {
 		account->priv->connecting = FALSE;
 		*info_result = EXCHANGE_ACCOUNT_UNKNOWN_ERROR;
+		g_mutex_unlock (account->priv->connect_lock);
 		return NULL; /* FIXME: what error has happened? */
 	}
 
@@ -1556,6 +1606,7 @@ exchange_account_connect (ExchangeAccount *account, const char *pword,
 
 	if (!setup_account_hierarchies (account)) {
 		*info_result = EXCHANGE_ACCOUNT_UNKNOWN_ERROR;
+		g_mutex_unlock (account->priv->connect_lock);
 		return NULL; /* FIXME: what error has happened? */
 	}
 
@@ -1600,6 +1651,7 @@ exchange_account_connect (ExchangeAccount *account, const char *pword,
 			  G_CALLBACK (context_redirect), account);
 
 	g_signal_emit (account, signals[CONNECTED], 0, account->priv->ctx);
+	g_mutex_unlock (account->priv->connect_lock);
 	return account->priv->ctx;
 }
 
@@ -1777,7 +1829,10 @@ exchange_account_get_folder (ExchangeAccount *account,
 			     const char *path_or_uri)
 {
 	g_return_val_if_fail (EXCHANGE_IS_ACCOUNT (account), NULL);
-	
+
+	if (!path_or_uri)
+		return NULL;
+
 	return g_hash_table_lookup (account->priv->folders, path_or_uri);
 }
 
@@ -1824,7 +1879,10 @@ exchange_account_get_folders (ExchangeAccount *account)
 	g_return_val_if_fail (EXCHANGE_IS_ACCOUNT (account), NULL);
 
 	folders = g_ptr_array_new ();
-	g_hash_table_foreach (account->priv->folders, add_folder, folders);
+	if (account->priv->fresh_folders)
+		g_hash_table_foreach (account->priv->fresh_folders, add_folder, folders);
+	else
+		g_hash_table_foreach (account->priv->folders, add_folder, folders);
 
 	qsort (folders->pdata, folders->len,
 	       sizeof (EFolder *), folder_comparator);
@@ -1942,11 +2000,25 @@ exchange_account_folder_size_rename (ExchangeAccount *account,
 GtkListStore *
 exchange_account_folder_size_get_model (ExchangeAccount *account)
 {
-	g_return_if_fail (EXCHANGE_IS_ACCOUNT (account));
+	g_return_val_if_fail (EXCHANGE_IS_ACCOUNT (account), NULL);
 
 	return exchange_folder_size_get_model (account->priv->fsize);
 }
 
+char *
+exchange_account_get_authtype (ExchangeAccount *account)
+{
+	g_return_val_if_fail (EXCHANGE_IS_ACCOUNT (account), NULL);
+
+	if (account->priv->auth_pref == E2K_AUTOCONFIG_USE_BASIC)
+		return g_strdup ("Basic");
+	else if (account->priv->auth_pref == E2K_AUTOCONFIG_USE_NTLM)
+		return g_strdup ("NTLM");
+
+	return NULL;
+}
+
+
 /**
  * exchange_account_new:
  * @adata: an #EAccount
